#!/bin/bash 

if [ -f path.sh ]; then . path.sh; fi

silprob=0.5
silphone="SIL"

arpa_lm=$1
lang_dir=$2
lexicon_file=$3


[ ! -f $arpa_lm ] && echo No such file $arpa_lm && exit 1;

cp -r data/lang $lang_dir

(echo '<sil> SIL'; echo '<brh> BRH'; echo '<cgh> CGH'; echo '<nsn> NSN'; echo '<um> UM'; echo '<uhh> UHH';) | cat - ${lexicon_file} > $lang_dir/lexicon.txt 

perl -ane '@A=split(" ",$_); $w = shift @A; @A>0||die;                                                                                                                             
           if(@A==1) { print "$w $A[0]_S\n"; } else { print "$w $A[0]_B ";                                                                                                             
           for($n=1;$n<@A-1;$n++) { print "$A[$n]_I "; } print "$A[$n]_E\n"; } ' < $lang_dir/lexicon.txt > $lang_dir/lexicon1.txt

utils/add_lex_disambig.pl $lang_dir/lexicon1.txt $lang_dir/lexicon_disambig.txt

cat $lang_dir/lexicon1.txt | awk '{print $1}' | sort | uniq  | \
 awk 'BEGIN{print "<eps> 0";} {printf("%s %d\n", $1, NR);} END{printf("#0 %d\n", NR+1);} ' \
  > $lang_dir/words.txt 

ndisambig=`utils/add_lex_disambig.pl $lang_dir/lexicon1.txt $lang_dir/lexicon_disambig.txt`
ndisambig=$[$ndisambig+1]; 

phone_disambig_symbol=`grep \#0 $lang_dir/phones.txt | awk '{print $2}'`
word_disambig_symbol=`grep \#0 $lang_dir/words.txt | awk '{print $2}'`

echo $phone_disambig_symbol
echo $word_disambig_symbol

cat $lang_dir/words.txt | grep "<UNK>" | cut -f2 -d" " > $lang_dir/oov.int


utils/make_lexicon_fst.pl $lang_dir/lexicon_disambig.txt $silprob $silphone '#'$ndisambig | fstcompile --isymbols=$lang_dir/phones.txt --osymbols=$lang_dir/words.txt --keep_isymbols=false --keep_osymbols=false | fstaddselfloops  "echo $phone_disambig_symbol |" "echo $word_disambig_symbol |" | fstarcsort --sort_type=olabel > $lang_dir/L_disambig.fst



cat "$arpa_lm" | \
   grep -v '<s> <s>' | \
   grep -v '</s> <s>' | \
   grep -v '<UNK>' | \
   grep -v '<unk>' | \
   grep -v '</s> </s>' | \
   arpa2fst - | fstprint | \
   utils/remove_oovs.pl /dev/null | \
   utils/eps2disambig.pl | utils/s2eps.pl | fstcompile --isymbols=$lang_dir/words.txt \
     --osymbols=$lang_dir/words.txt  --keep_isymbols=false --keep_osymbols=false | \
    fstrmepsilon > $lang_dir/G.fst
#fstisstochastic $lang_dir/G.fst


echo  "Checking how stochastic G is (the first of these numbers should be small):"
fstisstochastic $lang_dir/G.fst 

## Check lexicon.
## just have a look and make sure it seems sane.
#echo "First few lines of lexicon FST:"
#fstprint   --isymbols=$lang_dir/phones.txt --osymbols=$lang_dir/words.txt $lang_dir/L.fst  | head

#echo Performing further checks

# Checking that G.fst is determinizable.
#fstdeterminize $lang_dir/G.fst /dev/null || echo Error determinizing G.

# Checking that L_disambig.fst is determinizable.
#fstdeterminize $lang_dir/L_disambig.fst /dev/null || echo Error determinizing L.

# Checking that disambiguated lexicon times G is determinizable
# Note: we do this with fstdeterminizestar not fstdeterminize, as
# fstdeterminize was taking forever (presumbaly relates to a bug
# in this version of OpenFst that makes determinization slow for
# some case).
#fsttablecompose $lang_dir/L_disambig.fst $lang_dir/G.fst | \
#   fstdeterminizestar >/dev/null || echo Error

# Checking that LG is stochastic:
#fsttablecompose $lang_dir/L_disambig.fst $lang_dir/G.fst | \
#   fstisstochastic || echo LG is not stochastic


#echo swbd_p1_format_data succeeded.

